@@ -1,6 +1,6 @@
     /*
 
-    $Id: kfax.h,v 1.46 2004/11/09 19:30:31 deller Exp $
+    $Id: kfax.h 465369 2005-09-29 14:33:08Z mueller $
 
     Requires the Qt widget libraries, available at no cost at
     http://www.troll.no
@@ -21,7 +21,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     */
 
