@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004 by Albert Astals Cid <tsdgeos@terra.es>            *
+ *   Copyright (C) 2004-2006 by Albert Astals Cid <tsdgeos@terra.es>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -9,11 +9,11 @@
 
 // qt/kde includes
 #include <qheader.h>
+#include <qvariant.h>
 #include <klocale.h>
 
 // local includes
 #include "toc.h"
-#include "core/document.h"
 #include "core/link.h"
 #include "core/page.h"
 
@@ -32,6 +32,7 @@ class TOCItem : public KListViewItem
             if ( e.hasAttribute( "Page" ) )
                 setText( 1, e.attribute( "Page" ) );
 #endif
+            setMultiLinesEnabled(true);
         }
 
         TOCItem( KListViewItem *parent, TOCItem *after, const QDomElement & e )
@@ -41,6 +42,7 @@ class TOCItem : public KListViewItem
             if ( e.hasAttribute( "Page" ) )
                 setText( 1, e.attribute( "Page" ) );
 #endif
+            setMultiLinesEnabled(true);
         }
 
         const QDomElement & element() const
@@ -65,7 +67,12 @@ TOC::TOC(QWidget *parent, KPDFDocument *document) : KListView(parent), m_documen
     setResizeMode(AllColumns);
     setAllColumnsShowFocus(true);
     connect(this, SIGNAL(clicked(QListViewItem *)), this, SLOT(slotExecuted(QListViewItem *)));
-    connect(this, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(slotExecuted(QListViewItem *)));	    
+    connect(this, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(slotExecuted(QListViewItem *)));
+}
+
+TOC::~TOC()
+{
+    m_document->removeObserver( this );
 }
 
 uint TOC::observerId() const
@@ -73,9 +80,9 @@ uint TOC::observerId() const
     return TOC_ID;
 }
 
-void TOC::notifySetup( const QValueVector< KPDFPage * > & pages, bool documentChanged )
+void TOC::notifySetup( const QValueVector< KPDFPage * > & /*pages*/, bool documentChanged )
 {
-    if ( !documentChanged || pages.size() < 1 )
+    if ( !documentChanged )
         return;
 
     // clear contents
@@ -115,6 +122,13 @@ void TOC::addChildren( const QDomNode & parentNode, KListViewItem * parentItem )
         // descend recursively and advance to the next node
         if ( e.hasChildNodes() )
             addChildren( n, currentItem );
+
+        // open/keep close the item
+        bool isOpen = false;
+        if ( e.hasAttribute( "Open" ) )
+            isOpen = QVariant( e.attribute( "Open" ) ).toBool();
+        currentItem->setOpen( isOpen );
+
         n = n.nextSibling();
     }
 }
@@ -130,25 +144,31 @@ void TOC::slotExecuted( QListViewItem *i )
     QString externalFileName = e.attribute( "ExternalFileName" );
     if ( !externalFileName.isEmpty() )
     {
-        KPDFLinkGoto link( externalFileName, DocumentViewport() );
+        KPDFLinkGoto link( externalFileName, getViewport( e ) );
         m_document->processLink( &link );
     }
     else
     {
-        if ( e.hasAttribute( "Viewport" ) )
-        {
-            // if the node has a viewport, set it
-            m_document->setViewport( DocumentViewport( e.attribute( "Viewport" ) ), TOC_ID );
-        }
-        else if ( e.hasAttribute( "ViewportName" ) )
-        {
-            // if the node references a viewport, get the reference and set it
-            const QString & page = e.attribute( "ViewportName" );
-            const QString & viewport = m_document->getMetaData( "NamedViewport", page );
-            if ( !viewport.isNull() )
-                m_document->setViewport( DocumentViewport( viewport ), TOC_ID );
-        }
+        m_document->setViewport( getViewport( e ), TOC_ID );
     }
 }
 
+DocumentViewport TOC::getViewport( const QDomElement &e ) const
+{
+    if ( e.hasAttribute( "Viewport" ) )
+    {
+        // if the node has a viewport, set it
+        return DocumentViewport( e.attribute( "Viewport" ) );
+    }
+    else if ( e.hasAttribute( "ViewportName" ) )
+    {
+        // if the node references a viewport, get the reference and set it
+        const QString & page = e.attribute( "ViewportName" );
+        const QString & viewport = m_document->getMetaData( "NamedViewport", page );
+        if ( !viewport.isNull() )
+            return DocumentViewport( viewport );
+    }
+    return DocumentViewport();
+}
+
 #include "toc.moc"
