@@ -22,7 +22,6 @@
 #include "Stream.h"
 #include "Error.h"
 #include "Function.h"
-#include "UGString.h"
 
 //------------------------------------------------------------------------
 // Function
@@ -191,10 +190,11 @@ SampledFunction::SampledFunction(Object *funcObj, Dict *dict) {
   Object obj1, obj2;
   Guint buf, bitMask;
   int bits;
-  int s;
+  Guint s;
   int i;
 
   samples = NULL;
+  sBuf = NULL;
   ok = gFalse;
 
   //----- initialize the generic stuff
@@ -205,6 +205,14 @@ SampledFunction::SampledFunction(Object *funcObj, Dict *dict) {
     error(-1, "Type 0 function is missing range");
     goto err1;
   }
+  if (m > sampledFuncMaxInputs) {
+    error(-1, "Sampled functions with more than %d inputs are unsupported",
+	  sampledFuncMaxInputs);
+    goto err1;
+  }
+
+  //----- buffer
+  sBuf = (double *)gmallocn(1 << m, sizeof(double));
 
   //----- get the stream
   if (!funcObj->isStream()) {
@@ -240,7 +248,7 @@ SampledFunction::SampledFunction(Object *funcObj, Dict *dict) {
     goto err2;
   }
   sampleBits = obj1.getInt();
-  sampleMul = 1.0 / (double)((1 << sampleBits) - 1);
+  sampleMul = 1.0 / (pow(2.0, (double)sampleBits) - 1);
   obj1.free();
 
   //----- Encode
@@ -348,12 +356,16 @@ SampledFunction::~SampledFunction() {
   if (samples) {
     gfree(samples);
   }
+  if (sBuf) {
+    gfree(sBuf);
+  }
 }
 
 SampledFunction::SampledFunction(SampledFunction *func) {
   memcpy(this, func, sizeof(SampledFunction));
   samples = (double *)gmallocn(nSamples, sizeof(double));
   memcpy(samples, func->samples, nSamples * sizeof(double));
+  sBuf = (double *)gmallocn(1 << m, sizeof(double));
 }
 
 void SampledFunction::transform(double *in, double *out) {
@@ -361,7 +373,6 @@ void SampledFunction::transform(double *in, double *out) {
   int e[funcMaxInputs][2];
   double efrac0[funcMaxInputs];
   double efrac1[funcMaxInputs];
-  double s[1 << funcMaxInputs];
   int i, j, k, idx, t;
 
   // map input values into sample array
@@ -390,18 +401,18 @@ void SampledFunction::transform(double *in, double *out) {
       for (k = 0, t = j; k < m; ++k, t >>= 1) {
 	idx += idxMul[k] * (e[k][t & 1]);
       }
-      s[j] = samples[idx];
+      sBuf[j] = samples[idx];
     }
 
     // do m sets of interpolations
     for (j = 0, t = (1<<m); j < m; ++j, t >>= 1) {
       for (k = 0; k < t; k += 2) {
-	s[k >> 1] = efrac0[j] * s[k] + efrac1[j] * s[k+1];
+	sBuf[k >> 1] = efrac0[j] * sBuf[k] + efrac1[j] * sBuf[k+1];
       }
     }
 
     // map output value to range
-    out[i] = s[0] * (decode[i][1] - decode[i][0]) + decode[i][0];
+    out[i] = sBuf[0] * (decode[i][1] - decode[i][0]) + decode[i][0];
     if (out[i] < range[i][0]) {
       out[i] = range[i][0];
     } else if (out[i] > range[i][1]) {
@@ -541,6 +552,7 @@ StitchingFunction::StitchingFunction(Object * /*funcObj*/, Dict *dict) {
   funcs = NULL;
   bounds = NULL;
   encode = NULL;
+  scale = NULL;
 
   //----- initialize the generic stuff
   if (!init(dict)) {
@@ -560,6 +572,7 @@ StitchingFunction::StitchingFunction(Object * /*funcObj*/, Dict *dict) {
   funcs = (Function **)gmallocn(k, sizeof(Function *));
   bounds = (double *)gmallocn(k + 1, sizeof(double));
   encode = (double *)gmallocn(2 * k, sizeof(double));
+  scale = (double *)gmallocn(k, sizeof(double));
   for (i = 0; i < k; ++i) {
     funcs[i] = NULL;
   }
@@ -610,6 +623,17 @@ StitchingFunction::StitchingFunction(Object * /*funcObj*/, Dict *dict) {
   }
   obj1.free();
 
+  //----- pre-compute the scale factors
+  for (i = 0; i < k; ++i) {
+    if (bounds[i] == bounds[i+1]) {
+      // avoid a divide-by-zero -- in this situation, function i will
+      // never be used anyway
+      scale[i] = 0;
+    } else {
+      scale[i] = (encode[2*i+1] - encode[2*i]) / (bounds[i+1] - bounds[i]);
+    }
+  }
+
   ok = gTrue;
   return;
 
@@ -631,6 +655,8 @@ StitchingFunction::StitchingFunction(StitchingFunction *func) {
   memcpy(bounds, func->bounds, (k + 1) * sizeof(double));
   encode = (double *)gmallocn(2 * k, sizeof(double));
   memcpy(encode, func->encode, 2 * k * sizeof(double));
+  scale = (double *)gmallocn(k, sizeof(double));
+  memcpy(scale, func->scale, k * sizeof(double));
   ok = gTrue;
 }
 
@@ -647,6 +673,7 @@ StitchingFunction::~StitchingFunction() {
   gfree(funcs);
   gfree(bounds);
   gfree(encode);
+  gfree(scale);
 }
 
 void StitchingFunction::transform(double *in, double *out) {
@@ -665,8 +692,7 @@ void StitchingFunction::transform(double *in, double *out) {
       break;
     }
   }
-  x = encode[2*i] + ((x - bounds[i]) / (bounds[i+1] - bounds[i])) *
-                    (encode[2*i+1] - encode[2*i]);
+  x = encode[2*i] + (x - bounds[i]) * scale[i];
   funcs[i]->transform(&x, out);
 }
 
@@ -723,7 +749,7 @@ enum PSOp {
 // Note: 'if' and 'ifelse' are parsed separately.
 // The rest are listed here in alphabetical order.
 // The index in this table is equivalent to the entry in PSOp.
-const char *psOpNames[] = {
+char *psOpNames[] = {
   "abs",
   "add",
   "and",
@@ -1181,14 +1207,25 @@ GBool PostScriptFunction::parseCode(Stream *str, int *codePtr) {
 GString *PostScriptFunction::getToken(Stream *str) {
   GString *s;
   int c;
+  GBool comment;
 
   s = new GString();
-  do {
-    c = str->getChar();
-    if (c != EOF) {
-      codeString->append(c);
+  comment = gFalse;
+  while (1) {
+    if ((c = str->getChar()) == EOF) {
+      break;
     }
-  } while (c != EOF && isspace(c));
+    codeString->append(c);
+    if (comment) {
+      if (c == '\x0a' || c == '\x0d') {
+	comment = gFalse;
+      }
+    } else if (c == '%') {
+      comment = gTrue;
+    } else if (!isspace(c)) {
+      break;
+    }
+  }
   if (c == '{' || c == '}') {
     s->append((char)c);
   } else if (isdigit(c) || c == '.' || c == '-') {
